@@ -1,6 +1,6 @@
 /*
 * ModSecurity for Apache 2.x, http://www.modsecurity.org/
-* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+* Copyright (c) 2004-2013 Trustwave Holdings, Inc. (http://www.trustwave.com/)
 *
 * You may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
@@ -355,6 +355,8 @@ struct modsec_rec {
 
     apr_size_t           msc_reqbody_no_files_length;
 
+    char                *multipart_filename;
+    char                *multipart_name;
     multipart_data      *mpd;                        /* MULTIPART processor data structure */
 
     xml_data            *xml;                        /* XML processor data structure       */
@@ -440,6 +442,8 @@ struct modsec_rec {
     lua_State           *L;
     #endif
 #endif
+
+    int                 msc_sdbm_delete_error;
 };
 
 struct directory_config {
@@ -468,6 +472,7 @@ struct directory_config {
 
     int                  cookie_format;
     int                  argument_separator;
+    const char           *cookiev0_separator;
 
     int                  rule_inheritance;
     apr_array_header_t  *rule_exceptions;
@@ -571,9 +576,12 @@ struct directory_config {
     /* Collection timeout */
     int col_timeout;
 
+    /* hash of ids */
+    apr_hash_t          *rule_id_htab;
+
     /* Hash */
     apr_array_header_t  *hash_method;
-    const char *crypto_key;
+    const char          *crypto_key;
     int                 crypto_key_len;
     const char          *crypto_param_name;
     int                 hash_is_enabled;
@@ -589,6 +597,9 @@ struct directory_config {
     int                 crypto_hash_location_pm;
     int                 crypto_hash_iframesrc_pm;
     int                 crypto_hash_framesrc_pm;
+
+    /* xml */
+    int                 xml_external_entity;
 };
 
 struct error_message_t {
