@@ -1,6 +1,6 @@
 /*
 * ModSecurity for Apache 2.x, http://www.modsecurity.org/
-* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+* Copyright (c) 2004-2013 Trustwave Holdings, Inc. (http://www.trustwave.com/)
 *
 * You may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
@@ -229,7 +229,11 @@ static char *construct_auditlog_filename(apr_pool_t *mp, const char *uniqueid) {
  * as an audit log boundary.
  */
 static char *create_auditlog_boundary(request_rec *r) {
+#ifdef LINUX_S390
+    int data = swap_int32(rand());
+#else
     unsigned long data = rand();
+#endif
     /* Do note that I tried using apr_generate_random_bytes but it turned
      * out to be terribly slow for some reason. Needs further investigation.
      */
@@ -930,7 +934,7 @@ void sec_audit_logger(modsec_rec *msr) {
 
         /* Apache error messages */
         for(i = 0; i < msr->error_messages->nelts; i++) {
-            error_message *em = (((error_message**)msr->error_messages->elts)[i]);
+            error_message_t *em = (((error_message_t **)msr->error_messages->elts)[i]);
             text = apr_psprintf(msr->mp, "Apache-Error: %s\n",
                 format_error_log_message(msr->mp, em));
             sec_auditlog_write(msr, text, strlen(text));
@@ -1050,6 +1054,41 @@ void sec_audit_logger(modsec_rec *msr) {
                 msr->userid == NULL ? "-" : log_escape(msr->mp, msr->userid));
             sec_auditlog_write(msr, text, strlen(text));
         }
+
+        if ( ((msr->txcfg->sensor_id != NULL)&&(strcmp(msr->txcfg->sensor_id, "default") != 0)))
+        {
+            text = apr_psprintf(msr->mp, "Sensor-Id: \"%s\"\n",
+                msr->txcfg->sensor_id == NULL ? "-" : log_escape(msr->mp, msr->txcfg->sensor_id)),
+            sec_auditlog_write(msr, text, strlen(text));
+        }
+
+
+        if (msr->txcfg->is_enabled > 0) {
+            text = apr_psprintf(msr->mp, "Engine-Mode: \"%s\"\n",
+                msr->txcfg->is_enabled == 1 ? "DETECTION_ONLY" : "ENABLED"),
+            sec_auditlog_write(msr, text, strlen(text));
+        }
+
+        /* Rule performance time */
+        if(msr->txcfg->max_rule_time > 0)   {
+            const apr_array_header_t *tarr;
+            const apr_table_entry_t *telts;
+
+            tarr = apr_table_elts(msr->perf_rules);
+            telts = (const apr_table_entry_t*)tarr->elts;
+
+            if (tarr->nelts > 0) {
+                text = apr_psprintf(msr->mp, "Rules-Performance-Info: ");
+                sec_auditlog_write(msr, text, strlen(text));
+            }
+
+            for(i = 0; i < tarr->nelts; i++) {
+                text = apr_psprintf(msr->mp, "%s\"%s=%s\"%s", ((i == 0) ? "" : ", "),
+                    log_escape(msr->mp, telts[i].key), log_escape(msr->mp, telts[i].val), ((i == (tarr->nelts - 1)) ? ".\n" : ""));
+                sec_auditlog_write(msr, text, strlen(text));
+            }
+        }
+
     }
 
     /* AUDITLOG_PART_UPLOADS */
