@@ -1,6 +1,6 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+ * Copyright (c) 2004-2013 Trustwave Holdings, Inc. (http://www.trustwave.com/)
  *
  * You may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
@@ -14,10 +14,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#if APR_HAVE_STDINT_H
 #include <stdint.h>
+#endif
 #include <string.h>
-#if !defined(WIN32) || !defined(WINNT)
+#if APR_HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#if APR_HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
 #include "apr_lib.h"
