@@ -1,6 +1,6 @@
 /*
 * ModSecurity for Apache 2.x, http://www.modsecurity.org/
-* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+* Copyright (c) 2004-2013 Trustwave Holdings, Inc. (http://www.trustwave.com/)
 *
 * You may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
@@ -38,10 +38,11 @@
 #include "ap_config.h"
 #include "http_config.h"
 
+#include "api.h"
 
 extern void *modsecLogObj;
 extern void (*modsecLogHook)(void *obj, int level, char *str);
-
+extern int (*modsecDropAction)(request_rec *r);
 apr_status_t (*modsecReadBody)(request_rec *r, char *buf, unsigned int length, unsigned int *readcnt, int *is_eos);
 apr_status_t (*modsecReadResponse)(request_rec *r, char *buf, unsigned int length, unsigned int *readcnt, int *is_eos);
 apr_status_t (*modsecWriteBody)(request_rec *r, char *buf, unsigned int length);
@@ -143,17 +144,17 @@ server_rec *modsecInit()    {
     server->server_scheme = "";
     server->timeout = 60 * 1000000;// 60 seconds
     server->wild_names = NULL;
-	server->is_virtual = 0;
+    server->is_virtual = 0;
 
     ap_server_config_defines = apr_array_make(pool, 1, sizeof(char *));
 
     // here we should add scoreboard handling for multiple processes and threads
-	//
+    //
     ap_scoreboard_image = (scoreboard *)apr_palloc(pool, sizeof(scoreboard));
 
-	memset(ap_scoreboard_image, 0, sizeof(scoreboard));
+    memset(ap_scoreboard_image, 0, sizeof(scoreboard));
 
-	// ----------
+    // ----------
 
     security2_module.module_index = 0;
 
@@ -165,57 +166,73 @@ server_rec *modsecInit()    {
     return server;
 }
 
-apr_status_t ap_http_in_filter(ap_filter_t *f, apr_bucket_brigade *b,
+apr_status_t ap_http_in_filter(ap_filter_t *f, apr_bucket_brigade *bb_out,
         ap_input_mode_t mode, apr_read_type_e block,
         apr_off_t readbytes)    {
     char *tmp = NULL;
     apr_bucket *e = NULL;
     unsigned int readcnt = 0;
     int is_eos = 0;
+    apr_bucket_brigade *bb_in;
+    apr_bucket *after;
+    apr_status_t rv;
 
-    if(modsecReadBody == NULL)
-        return AP_NOBODY_READ;
+    bb_in = modsecGetBodyBrigade(f->r);
 
-    tmp = (char *)apr_palloc(f->r->pool, readbytes);
-    modsecReadBody(f->r, tmp, readbytes, &readcnt, &is_eos);
+    /* use request brigade */
+    if (bb_in != NULL) {
+        if (!APR_BUCKET_IS_EOS(APR_BRIGADE_LAST(bb_in))) {
+            e = apr_bucket_eos_create(f->c->bucket_alloc);
+            APR_BRIGADE_INSERT_TAIL(bb_in, e);
+        }
 
-    e = apr_bucket_pool_create(tmp, readcnt, f->r->pool, f->c->bucket_alloc);
-    APR_BRIGADE_INSERT_TAIL(b, e);
+        rv = apr_brigade_partition(bb_in, readbytes, &after);
+        if (rv != APR_SUCCESS && rv != APR_INCOMPLETE) {
+            return rv;
+        }
+        
+        for (e = APR_BRIGADE_FIRST(bb_in); e != after; e = APR_BRIGADE_FIRST(bb_in)) {
+            APR_BUCKET_REMOVE(e);
+            APR_BRIGADE_INSERT_TAIL(bb_out, e);
+        }
 
-    if(is_eos)  {
-        e = apr_bucket_eos_create(f->c->bucket_alloc);
-        APR_BRIGADE_INSERT_TAIL(b, e);
+        return APR_SUCCESS;
     }
 
+    /* call the callback */
+    if(modsecReadBody != NULL) {
+
+        tmp = (char *)apr_palloc(f->r->pool, readbytes);
+        modsecReadBody(f->r, tmp, readbytes, &readcnt, &is_eos);
+
+        e = apr_bucket_pool_create(tmp, readcnt, f->r->pool, f->c->bucket_alloc);
+        APR_BRIGADE_INSERT_TAIL(bb_out, e);
+
+        if(is_eos)  {
+            e = apr_bucket_eos_create(f->c->bucket_alloc);
+            APR_BRIGADE_INSERT_TAIL(bb_out, e);
+        }
+        return APR_SUCCESS;
+    }
+
+    /* cannot read request body */
+    e = apr_bucket_eos_create(f->c->bucket_alloc);
+    APR_BRIGADE_INSERT_TAIL(bb_out, e);
+
     return APR_SUCCESS;
 }
 
 apr_status_t ap_http_out_filter(ap_filter_t *f, apr_bucket_brigade *b)  {
-    modsec_rec *msr = (modsec_rec *)f->ctx;
     apr_status_t rc;
+    apr_bucket_brigade *bb_out = (apr_bucket_brigade *)f->ctx;
 
-    // is there a way to tell whether the response body was modified or not?
-    //
-    if((msr->txcfg->content_injection_enabled || msr->content_prepend_len != 0 || msr->content_append_len != 0)
-            && modsecWriteResponse != NULL && msr->txcfg->resbody_access)   {
-        char *data = NULL;
-        apr_size_t length;
-
-        rc = apr_brigade_pflatten(msr->of_brigade, &data, &length, msr->mp);
-
-        if (rc != APR_SUCCESS) {
-            msr_log(msr, 1, "Output filter: Failed to flatten brigade (%d): %s", rc,
-                    get_apr_error(msr->mp, rc));
-            return -1;
-        }
-
-        modsecWriteResponse(msr->r, data, msr->stream_output_length);
-    }
-
+    APR_BRIGADE_CONCAT(bb_out, b);
     return APR_SUCCESS;
 }
 
 void modsecTerminate()  {
+    apr_pool_destroy(pool);
+    pool = NULL;
     apr_terminate();
 }
 
@@ -233,13 +250,44 @@ directory_config *modsecGetDefaultConfig()  {
     return (directory_config *)security2_module.create_dir_config(pool, NULL);
 }
 
-const char *modsecProcessConfig(directory_config *config, const char *dir)  {
+const char *modsecProcessConfig(directory_config *config, const char *file, const char *dir)  {
     apr_pool_t *ptemp = NULL;
     const char *err;
+	apr_status_t status;
+	const char *rootpath, *incpath;
 
-    apr_pool_create(&ptemp, pool);
+	if(dir == NULL || strlen(dir) == 0)
+#ifdef	WIN32
+		dir = "\\";
+#else
+		dir = "/";
+#endif
 
-    err = process_command_config(server, config, pool, ptemp, dir);
+	incpath = file;
+
+	/* locate the start of the directories proper */
+	status = apr_filepath_root(&rootpath, &incpath, APR_FILEPATH_TRUENAME | APR_FILEPATH_NATIVE, pool);
+
+	/* we allow APR_SUCCESS and APR_EINCOMPLETE */
+	if (APR_ERELATIVE == status) {
+		int li = strlen(dir) - 1;
+
+		if(dir[li] != '/' && dir[li] != '\\')
+#ifdef	WIN32
+			file = apr_pstrcat(pool, dir, "\\", file, NULL);
+#else
+			file = apr_pstrcat(pool, dir, "/", file, NULL);
+#endif
+		else
+			file = apr_pstrcat(pool, dir, file, NULL);
+	}
+	else if (APR_EBADPATH == status) {
+		return apr_pstrcat(pool, "Config file has a bad path, ", file, NULL);
+	}
+
+	apr_pool_create(&ptemp, pool);
+
+    err = process_command_config(server, config, pool, ptemp, file);
 
     apr_pool_destroy(ptemp);
 
@@ -267,7 +315,7 @@ conn_rec *modsecNewConnection() {
 
     apr_pool_create(&pc, pool);
 
-    c = apr_palloc(pc, sizeof(conn_rec));
+    c = apr_pcalloc(pc, sizeof(conn_rec));
 
     c->base_server = server;
     c->id = 1;
@@ -300,7 +348,7 @@ request_rec *modsecNewRequest(conn_rec *connection, directory_config *config)
 
     apr_pool_create(&pr, connection->pool);
 
-    r = apr_palloc(pr, sizeof(request_rec));
+    r = apr_pcalloc(pr, sizeof(request_rec));
 
     r->connection = connection;
     r->server = server;
@@ -383,17 +431,25 @@ static modsec_rec *retrieve_msr(request_rec *r) {
     return NULL;
 }
 
-int modsecProcessRequest(request_rec *r)    {
+int modsecProcessRequestHeaders(request_rec *r) {
+    return hookfn_post_read_request(r);
+}
+
+int modsecProcessRequestBody(request_rec *r) {
     int status = DECLINED;
     modsec_rec *msr = NULL;
 
     ap_filter_t *f = ap_add_input_filter("HTTP_IN", NULL, r, r->connection);
+    apr_bucket_brigade* bb_out;
 
-    status = hookfn_post_read_request(r);
     status = hookfn_fixups(r);
 
     ap_remove_input_filter(f);
 
+	if (status != DECLINED) {
+		return status;
+	}
+
     hookfn_insert_filter(r);
 
     /* Find the transaction context first. */
@@ -402,6 +458,16 @@ int modsecProcessRequest(request_rec *r)    {
     if (msr == NULL)
         return status;
 
+    bb_out = modsecGetBodyBrigade(r);
+    if (bb_out) {
+        (void) apr_brigade_cleanup(bb_out);
+        status = ap_get_brigade(r->input_filters, bb_out, AP_MODE_READBYTES, APR_BLOCK_READ, -1);
+        if (status == APR_SUCCESS) {
+            return DECLINED;
+        }
+        return status;
+    }
+
     if(msr->stream_input_data != NULL && modsecWriteBody != NULL)
     {
         // target is responsible for copying the data into correctly managed buffer
@@ -423,74 +489,149 @@ int modsecProcessRequest(request_rec *r)    {
     return status;
 }
 
+void modsecSetConfigForIISRequestBody(request_rec *r)
+{
+    modsec_rec *msr = retrieve_msr(r);
+
+    if(msr == NULL || msr->txcfg == NULL)
+        return;
+
+    if(msr->txcfg->reqbody_access)
+        msr->txcfg->stream_inbody_inspection = 1;
+}
+
+int modsecIsRequestBodyAccessEnabled(request_rec *r)
+{
+    modsec_rec *msr = retrieve_msr(r);
+
+    if(msr == NULL || msr->txcfg == NULL)
+        return 0;
+
+    return msr->txcfg->reqbody_access;
+}
+
 int modsecIsResponseBodyAccessEnabled(request_rec *r)
 {
-	modsec_rec *msr = retrieve_msr(r);
+    modsec_rec *msr = retrieve_msr(r);
 
-	if(msr == NULL || msr->txcfg == NULL)
-		return 0;
+    if(msr == NULL || msr->txcfg == NULL)
+        return 0;
 
-	return msr->txcfg->resbody_access;
+    return msr->txcfg->resbody_access;
 }
 
 int modsecProcessResponse(request_rec *r)   {
-    int status = DECLINED;
+    int status;
+    modsec_rec *msr;
+    apr_bucket *e;
+    ap_filter_t *f;
+    apr_bucket_brigade *bb_in, *bb_out, *bb;
 
-    if(r->output_filters != NULL)   {
-        modsec_rec *msr = (modsec_rec *)r->output_filters->ctx;
-        char buf[8192];
-        char *tmp = NULL;
-        apr_bucket *e = NULL;
+    if(r->output_filters == NULL) {
+        return DECLINED;
+    }
+
+    msr = (modsec_rec *)r->output_filters->ctx;
+    if (msr == NULL) {
+        ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO, 0, r->server,
+                "ModSecurity: Internal Error: msr is null in output filter.");
+        ap_remove_output_filter(r->output_filters);
+        return APR_EGENERAL;
+    }
+
+    msr->r = r;
+
+    /* create input response brigade */
+    bb_in = apr_brigade_create(msr->mp, r->connection->bucket_alloc);
+
+    if (bb_in == NULL) {
+        msr_log(msr, 1, "Process response: Failed to create brigade.");
+        return APR_EGENERAL;
+    }
+
+    /* get input response brigade */
+    bb = modsecGetResponseBrigade(r);
+    if (bb != NULL) {
+        APR_BRIGADE_CONCAT(bb_in, bb);
+        if (!APR_BUCKET_IS_EOS(APR_BRIGADE_LAST(bb_in))) {
+            e = apr_bucket_eos_create(bb_in->bucket_alloc);
+            APR_BRIGADE_INSERT_TAIL(bb_in, e);
+        }
+    } else if (modsecReadResponse != NULL) {
         unsigned int readcnt = 0;
         int is_eos = 0;
-        ap_filter_t *f = NULL;
-        apr_bucket_brigade *bb = NULL;
-
-        if (msr == NULL) {
-            ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO, 0, r->server,
-                    "ModSecurity: Internal Error: msr is null in output filter.");
-            ap_remove_output_filter(r->output_filters);
-            return send_error_bucket(msr, r->output_filters, HTTP_INTERNAL_SERVER_ERROR);
-        }
-
-        bb = apr_brigade_create(msr->mp, r->connection->bucket_alloc);
-
-        if (bb == NULL) {
-            msr_log(msr, 1, "Process response: Failed to create brigade.");
-            return -1;
-        }
-
-        msr->r = r;
-
-        if(modsecReadResponse == NULL)
-            return AP_NOBODY_WROTE;
-
-        f = ap_add_output_filter("HTTP_OUT", msr, r, r->connection);
-
+        char buf[8192];
         while(!is_eos)  {
             modsecReadResponse(r, buf, 8192, &readcnt, &is_eos);
 
             if(readcnt > 0) {
-                tmp = (char *)apr_palloc(r->pool, readcnt);
+                char *tmp = (char *)apr_palloc(r->pool, readcnt);
                 memcpy(tmp, buf, readcnt);
-
                 e = apr_bucket_pool_create(tmp, readcnt, r->pool, r->connection->bucket_alloc);
-                APR_BRIGADE_INSERT_TAIL(bb, e);
-            }
-
-            if(is_eos)  {
-                e = apr_bucket_eos_create(r->connection->bucket_alloc);
-
-                APR_BRIGADE_INSERT_TAIL(bb, e);
+                APR_BRIGADE_INSERT_TAIL(bb_in, e);
             }
         }
 
-        status = ap_pass_brigade(r->output_filters, bb);
+        e = apr_bucket_eos_create(r->connection->bucket_alloc);
+        APR_BRIGADE_INSERT_TAIL(bb_in, e);
+    } else {
+        /* cannot read response body process header only */
 
-        ap_remove_output_filter(f);
+        e = apr_bucket_eos_create(r->connection->bucket_alloc);
+        APR_BRIGADE_INSERT_TAIL(bb_in, e);
     }
 
-    return status;
+    bb_out = bb ? bb : apr_brigade_create(msr->mp, r->connection->bucket_alloc);
+
+    if (bb_out == NULL) {
+        msr_log(msr, 1, "Process response: Failed to create brigade.");
+        return APR_EGENERAL;
+    }
+
+    /* concat output bucket to bb_out */
+    f = ap_add_output_filter("HTTP_OUT", bb_out, r, r->connection);
+    status = ap_pass_brigade(r->output_filters, bb_in);
+    ap_remove_output_filter(f);
+
+    if (status == APR_EGENERAL) {
+        /* retrive response status from bb_out */
+        for(e = APR_BRIGADE_FIRST(bb_out);
+                e != APR_BRIGADE_SENTINEL(bb_out);
+                e = APR_BUCKET_NEXT(e)) {
+            if (AP_BUCKET_IS_ERROR(e)) {
+                return ((ap_bucket_error*) e->data)->status;
+            }
+        }
+        return APR_EGENERAL;
+    }
+
+    if (status != DECLINED) {
+        return status;
+    }
+
+    /* copy bb_out */
+    // is there a way to tell whether the response body was modified or not?
+    if (modsecWriteResponse != NULL
+            && (msr->txcfg->content_injection_enabled || msr->content_prepend_len != 0 || msr->content_append_len != 0)
+            && msr->txcfg->resbody_access)   {
+
+        char *data = NULL;
+        apr_size_t length;
+
+        status = apr_brigade_pflatten(msr->of_brigade, &data, &length, msr->mp);
+
+        if (status != APR_SUCCESS) {
+            msr_log(msr, 1, "Output filter: Failed to flatten brigade (%d): %s", status,
+                    get_apr_error(msr->mp, status));
+            return APR_EGENERAL;
+        }
+
+        if ( modsecWriteResponse(msr->r, data, msr->stream_output_length) != APR_SUCCESS) {
+            return APR_EGENERAL;
+        }
+    }
+    
+    return DECLINED;
 }
 
 int modsecFinishRequest(request_rec *r) {
@@ -528,3 +669,7 @@ void modsecSetWriteBody(apr_status_t (*func)(request_rec *r, char *buf, unsigned
 void modsecSetWriteResponse(apr_status_t (*func)(request_rec *r, char *buf, unsigned int length))   {
     modsecWriteResponse = func;
 }
+
+void modsecSetDropAction(int (*func)(request_rec *r)) {
+    modsecDropAction = func;
+}
